;; -*- no-byte-compile: t; lexical-binding: nil -*-
(define-package "docker" "2.5.0"
  "Interface to Docker."
  '((aio       "1.0")
    (dash      "2.19.1")
    (emacs     "28.1")
    (s         "1.13.0")
    (tablist   "1.1")
    (transient "0.4.3"))
  :url "https://github.com/Silex/docker.el"
  :commit "916686b86e83a3bd2281fbc5e6f98962aa747429"
  :revdesc "2.5.0-0-g916686b86e83"
  :keywords '("filename" "convenience")
  :authors '(("Philippe Vaucher" . "philippe.vaucher@gmail.com"))
  :maintainers '(("Philippe Vaucher" . "philippe.vaucher@gmail.com")))
